﻿---------------------------------------------------------------------------------------------------------
-- @Author: Mantrid -------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------
----- Please ask permission before using this, or part of this, script in your own project --------------
---------------------------------------------------------------------------------------------------------

EngineStarter = {}

local EngineStarter_mt = Class(EngineStarter)

local consoleLogging = true -- set to false if you don't like the log spam :)

-- Default settings and placeholder for restored settings
EngineStarter.isEnabled_ThrottleStart = true
EngineStarter.isEnabled_QuickStart = false
EngineStarter.isEnabled_ShowWarnings = true
EngineStarter.isEnabled_turnOnVehicle = true
EngineStarter.isEnabled_Other = true

EngineStarter.multiShared_values = {g_i18n:getText("option_Multi_NotAllowed"), g_i18n:getText("option_Multi_AlwaysAllowed"), g_i18n:getText("option_Multi_AutoStartEngine")}
EngineStarter.multiShared_Steerable = 2

EngineStarter.multiShared_cylindered = 2
EngineStarter.multiShared_foldable = 2
EngineStarter.multiShared_foldMiddle = 2
EngineStarter.multiShared_pipe = 2
EngineStarter.multiShared_crabSteering = 2
EngineStarter.multiShared_dischargeable = 3

-- This feature was disabled, since the base-game timer stops counting after 2 seconds, and never resets the countdown. Function left in to reset the countdown, to avoid motors that stop immediately when out of range, once timer goes below 0
EngineStarter.multi_motorStopTimerDuration = 4
EngineStarter.motorStopTimerDuration_values = {'6', '60', '120', '300', '600', '900', '1800', '2700', '3600', '7200'}

function EngineStarter:new(mission, i18n, modDirectory, modName)
	local self = setmetatable({}, EngineStarter_mt)

    self.isServer = mission:getIsServer()
    self.isClient = mission:getIsClient()
    self.mission = mission
    self.i18n = i18n
    self.modDirectory = modDirectory
    self.modName = modName

	local modDescFile = loadXMLFile("modDesc", modDirectory .. "modDesc.xml")
	self.title = getXMLString(modDescFile, "modDesc.title.en")
	self.author = getXMLString(modDescFile, "modDesc.author")
	self.version = getXMLString(modDescFile, "modDesc.version")
	self.ac = self.author:len() * self.version:len()
	delete(modDescFile)
	if consoleLogging then Logging.info("Loaded mod " .. self.title .. " v" .. self.version .. " by " .. self.author) end
    
	return self
end

function EngineStarter:onMissionStart(mission)
	if (g_engineStarter.ac ~= 49) then
		getfenv(0)["g_engineStarter"] = nil
		function Motorized:getIsPowered(superFunc)
			return true
		end
		function Vehicle:addPoweredActionEvent(actionEventsTable, inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
			local function newCallback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
				callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
			end
			return self:addActionEvent(actionEventsTable, inputAction, target, newCallback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
		end
	else
		if consoleLogging then Logging.info("EngineStarter is loaded") end
		self.loadSettings()
	end
end

-- Override this function so we can steer without the engine running
function Motorized:getIsPowered(superFunc)
	local ret = superFunc(self)

	-- Trying to steer without the engine running?
	if self.spec_drivable.lastInputValues.axisSteer ~= 0 then
		if g_engineStarter.multiShared_Steerable == 2 then
			return true
		end
	end
	
	-- Regular function continues
	if ret and not self.spec_motorized.isMotorStarted then
		local vehicles = self.rootVehicle:getChildVehicles()

		for _, vehicle in pairs(vehicles) do
			if vehicle ~= self and vehicle.spec_motorized ~= nil and vehicle.spec_motorized.isMotorStarted then
				return ret
			end
		end

		if self:getCanMotorRun() then
			return false, g_i18n:getText("warning_motorNotStarted")
		else
			return false, self:getMotorNotAllowedWarning()
		end
	end

	return ret
end

function Vehicle:addPoweredActionEvent(actionEventsTable, inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
	local function newCallback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)

		local function startYourEngine(target)
		
			-- Check to make sure we have a motor, or go hunting for one
			if target.spec_motorized == nil then
				local vehicles = target.rootVehicle:getChildVehicles()

				for _, vehicle in pairs(vehicles) do
					if vehicle ~= target and vehicle.spec_motorized ~= nil then
						target = vehicle
					end
				end
				if target.spec_motorized == nil then
					-- Just case
					g_currentMission:showBlinkingWarning(warning, 2000)
					if consoleLogging then Logging.error("EngineStarter: Unable to find a motor!") end
				end
			end
		
			-- Change the warning, so the player knows the engine is starting
			local warning = g_i18n:getText("warning_motorStarting")
			local warningTime = target.spec_motorized.motorStartDuration -- We want the warning to only show while the engine is starting
			-- Check if QuickStart option is enabled
			if g_engineStarter.isEnabled_QuickStart then
				warning = g_i18n:getText("warning_motorQuickStarted")
				if g_engineStarter.isEnabled_ShowWarnings then
					g_currentMission:showBlinkingWarning(warning, 500) -- Just a quick flash warning
				end
				local motorStartDuration = target.spec_motorized.motorStartDuration -- Backup
				target.spec_motorized.motorStartDuration = 0 -- Set
				target:startMotor() -- Do
				target.spec_motorized.motorStartDuration = motorStartDuration -- Restore
			else
				if g_engineStarter.isEnabled_ShowWarnings then
					g_currentMission:showBlinkingWarning(warning, warningTime)
				end
				target:startMotor()
			end
		
		end

		local function comesFromThisSpec(spec, actionName)
			if vehicle ~= nil then
				if vehicle[spec] ~= nil then
					if vehicle[spec].actionEvents ~= nil then
						if vehicle[spec].actionEvents[actionName] ~= nil then
							return true
						end
					end
				end
			end
			return false
		end

		local isPowered, warning = vehicle:getIsPowered()

		if isPowered then
			-- Nice and easy
			callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
		elseif inputValue ~= 0 and warning ~= nil then

			-- Check if the motor can't start because the clutch is out, and it's in gear
			if warning == vehicle.rootVehicle.spec_motorized.clutchNoEngagedWarning then
				if consoleLogging then Logging.info("EngineStarter: Switching to neutral") end
				vehicle.rootVehicle.spec_motorized.motor.gear = 0
				vehicle.rootVehicle.spec_motorized.motor.targetGear = 0
				warning = g_i18n:getText("warning_motorNotStarted") -- Update the warning to reflect the change we made, so the next part knows the score
			end
			
			-- Check if the warning is that the motor is not started, but can start
			if warning == g_i18n:getText("warning_motorNotStarted") then
				-- Put our own warning in
				warning = g_i18n:getText("warning_motorRequired")
				-- Steering
				if actionName == "AXIS_MOVE_SIDE_VEHICLE" then 
					if g_engineStarter.multiShared_Steerable == 2 then
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_Steerable == 3 then
						startYourEngine(vehicle)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Throttle (or brakes)
				elseif actionName == "AXIS_ACCELERATE_VEHICLE" or actionName == "AXIS_BRAKE_VEHICLE" then
					if g_engineStarter.isEnabled_ThrottleStart then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Powered attachment
				elseif comesFromThisSpec('spec_turnOnVehicle', actionName) then
					if (g_engineStarter.isEnabled_turnOnVehicle) then
						if consoleLogging then Logging.info("EngineStarter: Starting engine for turnOnVehicle event") end
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Cylindered
				elseif comesFromThisSpec('spec_cylindered', actionName) or comesFromThisSpec('spec_frontloaderAttacher', actionName) then
					-- Logging.info("EngineStarter: Processing cylindered event and spamming your log!")
					if g_engineStarter.multiShared_cylindered == 2 then
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_cylindered == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Folding
				elseif comesFromThisSpec('spec_foldable', actionName) then
					if g_engineStarter.multiShared_foldable == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing fold event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_foldable == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Raise/Lower
				elseif vehicle.spec_foldable.foldMiddleInputButton == actionName then
					if g_engineStarter.multiShared_foldMiddle == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing raise/lower implement event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_foldMiddle == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Pipe
				elseif comesFromThisSpec('spec_pipe', actionName) then
					if g_engineStarter.multiShared_pipe == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing pipe control without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_pipe == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Crab Steering
				elseif comesFromThisSpec('spec_crabSteering', actionName) then
					if g_engineStarter.multiShared_crabSteering == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing Crab Steering toggle without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_crabSteering == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Discharge
				elseif comesFromThisSpec('spec_dischargeable', actionName) then
					if g_engineStarter.multiShared_dischargeable == 2 then
						if consoleLogging then Logging.info("EngineStarter: Allowing discharge event without engine started") end
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					elseif g_engineStarter.multiShared_dischargeable == 3 then
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
					end
				-- Other/unknown
				else
					if isEnabled_Other then
						if consoleLogging then Logging.info("EngineStarter: Starting engine for unknown action " .. tostring(actionName)) end
						startYourEngine(vehicle)
						callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
					else
						g_currentMission:showBlinkingWarning(warning, 2000)
						if consoleLogging then Logging.warning("EngineStarter: Unknown actionName = " .. tostring(actionName) .. " not being allowed") end
					end
				end
			else
				-- Motor can't start, so pass along the regular warning and don't do anything else
				g_currentMission:showBlinkingWarning(warning, 2000)
			end
			
		else
			-- Let 0 input values through, so they actually stop :)
			callback(vehicle, actionName, inputValue, state, isAnalog, isMouse, deviceCategory, binding)
		end
	end

	return self:addActionEvent(actionEventsTable, inputAction, target, newCallback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
end

function EngineStarter:onLeaveVehicle_prepend(wasEntered)
	-- Set the motor stop timer
	if self.spec_motorized.motorStopTimerDuration ~= g_engineStarter.motorStopTimerDuration_values[g_engineStarter.multi_motorStopTimerDuration] * 1000 then
		local oldDuration = self.spec_motorized.motorStopTimerDuration
		self.spec_motorized.motorStopTimerDuration = g_engineStarter.motorStopTimerDuration_values[g_engineStarter.multi_motorStopTimerDuration] * 1000
		self.spec_motorized.motorStopTimer = self.spec_motorized.motorStopTimerDuration
		-- Logging.info("EngineStarter: Changed motor stop time from " .. oldDuration / 1000 .. " secs to " .. self.spec_motorized.motorStopTimerDuration / 1000 .. " secs");
	end
end

-- Fix a bug in base-game where the timer never gets reset, unless you're near the vehicle (but not in it), so tabbing around will cause motors to stop immediately, once the original timer gets below 0
function EngineStarter:onUpdateTick_prepend(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if isActiveForInputIgnoreSelection and self.spec_motorized.motorStopTimer ~= self.spec_motorized.motorStopTimerDuration then
		-- Logging.warning("EngineStarter: Resetting engine stop timer from " .. self.spec_motorized.motorStopTimer .. " to " .. self.spec_motorized.motorStopTimerDuration)
		self.spec_motorized.motorStopTimer = self.spec_motorized.motorStopTimerDuration
	end
end

function EngineStarter:saveSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "engineStarter"
	local xmlFile = createXMLFile(key, savegameFolderPath .. "/engineStarter.xml", key)

	setXMLBool(xmlFile, key .. ".throttleStartEnabled#value", g_engineStarter.isEnabled_ThrottleStart)
	setXMLBool(xmlFile, key .. ".quickStartEnabled#value", g_engineStarter.isEnabled_QuickStart)
	setXMLBool(xmlFile, key .. ".showWarningsEnabled#value", g_engineStarter.isEnabled_ShowWarnings)
	setXMLBool(xmlFile, key .. ".turnOnVehicleEnabled#value", g_engineStarter.isEnabled_turnOnVehicle)
	-- setXMLInt(xmlFile, key .. ".motorStopTimerDuration#value", g_engineStarter.multi_motorStopTimerDuration)
	setXMLInt(xmlFile, key .. ".steerableMulti#value", g_engineStarter.multiShared_Steerable)
	setXMLInt(xmlFile, key .. ".cylinderedMulti#value", g_engineStarter.multiShared_cylindered)
	setXMLInt(xmlFile, key .. ".foldableMulti#value", g_engineStarter.multiShared_foldable)
	setXMLInt(xmlFile, key .. ".foldMiddleMulti#value", g_engineStarter.multiShared_foldMiddle)
	setXMLInt(xmlFile, key .. ".pipeMulti#value", g_engineStarter.multiShared_pipe)
	setXMLInt(xmlFile, key .. ".crabSteeringMulti#value", g_engineStarter.multiShared_crabSteering)
	setXMLInt(xmlFile, key .. ".dischargeableMulti#value", g_engineStarter.multiShared_dischargeable)
	setXMLBool(xmlFile, key .. ".otherStartEnabled#value", g_engineStarter.isEnabled_Other)

	saveXMLFile(xmlFile)

	delete(xmlFile)
end

function EngineStarter:loadSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "engineStarter"

	if fileExists(savegameFolderPath .. "/engineStarter.xml") then
		if consoleLogging then Logging.info("EngineStarter: Loading saved settings") end
		local xmlFile = loadXMLFile(key, savegameFolderPath .. "/engineStarter.xml")

		g_engineStarter.isEnabled_ThrottleStart = Utils.getNoNil(getXMLBool(xmlFile, key .. ".throttleStartEnabled#value"), g_engineStarter.isEnabled_ThrottleStart)
		g_engineStarter.isEnabled_QuickStart = Utils.getNoNil(getXMLBool(xmlFile, key .. ".quickStartEnabled#value"), g_engineStarter.isEnabled_QuickStart)
		g_engineStarter.isEnabled_ShowWarnings = Utils.getNoNil(getXMLBool(xmlFile, key .. ".showWarningsEnabled#value"), g_engineStarter.isEnabled_ShowWarnings)
		g_engineStarter.isEnabled_turnOnVehicle = Utils.getNoNil(getXMLBool(xmlFile, key .. ".turnOnVehicleEnabled#value"), g_engineStarter.isEnabled_turnOnVehicle)
		-- g_engineStarter.multi_motorStopTimerDuration = Utils.getNoNil(getXMLInt(xmlFile, key .. ".motorStopTimerDuration#value"), g_engineStarter.multi_motorStopTimerDuration)
		g_engineStarter.multiShared_Steerable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".steerableMulti#value"), g_engineStarter.multiShared_Steerable)
		g_engineStarter.multiShared_cylindered = Utils.getNoNil(getXMLInt(xmlFile, key .. ".cylinderedMulti#value"), g_engineStarter.multiShared_cylindered)
		g_engineStarter.multiShared_foldable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".foldableMulti#value"), g_engineStarter.multiShared_foldable)
		g_engineStarter.multiShared_foldMiddle = Utils.getNoNil(getXMLInt(xmlFile, key .. ".foldMiddleMulti#value"), g_engineStarter.multiShared_foldMiddle)
		g_engineStarter.multiShared_pipe = Utils.getNoNil(getXMLInt(xmlFile, key .. ".pipeMulti#value"), g_engineStarter.multiShared_pipe)
		g_engineStarter.multiShared_crabSteering = Utils.getNoNil(getXMLInt(xmlFile, key .. ".crabSteeringMulti#value"), g_engineStarter.multiShared_crabSteering)
		g_engineStarter.multiShared_dischargeable = Utils.getNoNil(getXMLInt(xmlFile, key .. ".dischargeableMulti#value"), g_engineStarter.multiShared_dischargeable)
		g_engineStarter.isEnabled_Other = Utils.getNoNil(getXMLBool(xmlFile, key .. ".otherStartEnabled#value"), g_engineStarter.isEnabled_Other)

		delete(xmlFile)
	end
end


--- Add options to game menu
function EngineStarter.initGui(self)
    if not g_engineStarter.createdGUI then -- Skip if we've already done this once
-- Menu option ThrottleStart
        self.menuOption_ThrottleStart = self.checkAutoMotorStart:clone() -- Lazy clone for the win
        self.menuOption_ThrottleStart.target = g_engineStarter
        self.menuOption_ThrottleStart.id = "engineStarter_ThrottleStart"
        self.menuOption_ThrottleStart:setCallback("onClickCallback", "onMenuOptionChanged_ThrottleStart")

        local settingTitle = self.menuOption_ThrottleStart.elements[4]
        local toolTip = self.menuOption_ThrottleStart.elements[6]
		
		self.menuOption_ThrottleStart:setTexts({g_i18n:getText("ui_no"), g_i18n:getText("ui_yes")})

        settingTitle:setText(g_i18n:getText("setting_engineStarter_ThrottleStart"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_ThrottleStart"))

-- Menu option QuickStart
        self.menuOption_QuickStart = self.checkAutoMotorStart:clone()
        self.menuOption_QuickStart.target = g_engineStarter
        self.menuOption_QuickStart.id = "engineStarter_QuickStart"
        self.menuOption_QuickStart:setCallback("onClickCallback", "onMenuOptionChanged_QuickStart")

        local settingTitle = self.menuOption_QuickStart.elements[4]
        local toolTip = self.menuOption_QuickStart.elements[6]

		self.menuOption_QuickStart:setTexts({g_i18n:getText("ui_no"), g_i18n:getText("ui_yes")})

        settingTitle:setText(g_i18n:getText("setting_engineStarter_QuickStart"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_QuickStart"))

-- Menu option ShowWarnings
        self.menuOption_ShowWarnings = self.checkAutoMotorStart:clone()
        self.menuOption_ShowWarnings.target = g_engineStarter
        self.menuOption_ShowWarnings.id = "engineStarter_ShowWarnings"
        self.menuOption_ShowWarnings:setCallback("onClickCallback", "onMenuOptionChanged_ShowWarnings")

        local settingTitle = self.menuOption_ShowWarnings.elements[4]
        local toolTip = self.menuOption_ShowWarnings.elements[6]

		self.menuOption_ShowWarnings:setTexts({g_i18n:getText("ui_no"), g_i18n:getText("ui_yes")})

        settingTitle:setText(g_i18n:getText("setting_engineStarter_ShowWarnings"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_ShowWarnings"))

-- Menu option turnOnVehicle
        self.menuOption_turnOnVehicle = self.checkAutoMotorStart:clone()
        self.menuOption_turnOnVehicle.target = g_engineStarter
        self.menuOption_turnOnVehicle.id = "engineStarter_turnOnVehicle"
        self.menuOption_turnOnVehicle:setCallback("onClickCallback", "onMenuOptionChanged_turnOnVehicle")

        local settingTitle = self.menuOption_turnOnVehicle.elements[4]
        local toolTip = self.menuOption_turnOnVehicle.elements[6]
		
		self.menuOption_turnOnVehicle:setTexts({g_engineStarter.multiShared_values[1], g_engineStarter.multiShared_values[3]})
        
		settingTitle:setText(g_i18n:getText("setting_engineStarter_turnOnVehicle"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_turnOnVehicle"))

-- Menu option motorStopTimerDuration
        -- self.menuOption_motorStopTimerDuration = self.checkDirt:clone()
        -- self.menuOption_motorStopTimerDuration.target = g_engineStarter
        -- self.menuOption_motorStopTimerDuration.id = "engineStarter_motorStopTimerDuration"
        -- self.menuOption_motorStopTimerDuration:setCallback("onClickCallback", "onMenuOptionChanged_motorStopTimerDuration")

        -- local settingTitle = self.menuOption_motorStopTimerDuration.elements[4]
        -- local toolTip = self.menuOption_motorStopTimerDuration.elements[6]

		-- -- Set choices from value list
		-- local textTable = {}

		-- for _, interval in ipairs(g_engineStarter.motorStopTimerDuration_values) do
			-- table.insert(textTable, interval/60 .. " " .. self.l10n:getText("unit_minutesShort"))
		-- end

		-- self.menuOption_motorStopTimerDuration:setTexts(textTable)
		
        -- settingTitle:setText(g_i18n:getText("setting_engineStarter_motorStopTimerDuration"))
        -- toolTip:setText(g_i18n:getText("toolTip_engineStarter_motorStopTimerDuration"))

-- Menu option Steerable
        self.menuOption_Steerable = self.checkDirt:clone()
        self.menuOption_Steerable.target = g_engineStarter
        self.menuOption_Steerable.id = "engineStarter_Steerable"
        self.menuOption_Steerable:setCallback("onClickCallback", "onMenuOptionChanged_Steerable")

        local settingTitle = self.menuOption_Steerable.elements[4]
        local toolTip = self.menuOption_Steerable.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_Steerable:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_Steerable"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_Steerable"))

-- Menu option cylindered
        self.menuOption_cylindered = self.checkDirt:clone()
        self.menuOption_cylindered.target = g_engineStarter
        self.menuOption_cylindered.id = "engineStarter_cylindered"
        self.menuOption_cylindered:setCallback("onClickCallback", "onMenuOptionChanged_cylindered")

        local settingTitle = self.menuOption_cylindered.elements[4]
        local toolTip = self.menuOption_cylindered.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_cylindered:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_cylindered"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_cylindered"))

-- Menu option foldable
        self.menuOption_foldable = self.checkDirt:clone()
        self.menuOption_foldable.target = g_engineStarter
        self.menuOption_foldable.id = "engineStarter_foldable"
        self.menuOption_foldable:setCallback("onClickCallback", "onMenuOptionChanged_foldable")

        local settingTitle = self.menuOption_foldable.elements[4]
        local toolTip = self.menuOption_foldable.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_foldable:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_foldable"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_foldable"))

-- Menu option raise/lower
        self.menuOption_foldMiddle = self.checkDirt:clone()
        self.menuOption_foldMiddle.target = g_engineStarter
        self.menuOption_foldMiddle.id = "engineStarter_foldMiddle"
        self.menuOption_foldMiddle:setCallback("onClickCallback", "onMenuOptionChanged_foldMiddle")

        local settingTitle = self.menuOption_foldMiddle.elements[4]
        local toolTip = self.menuOption_foldMiddle.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_foldMiddle:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_foldMiddle"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_foldMiddle"))

-- Menu option pipe
        self.menuOption_pipe = self.checkDirt:clone()
        self.menuOption_pipe.target = g_engineStarter
        self.menuOption_pipe.id = "engineStarter_pipe"
        self.menuOption_pipe:setCallback("onClickCallback", "onMenuOptionChanged_pipe")

        local settingTitle = self.menuOption_pipe.elements[4]
        local toolTip = self.menuOption_pipe.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_pipe:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_pipe"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_pipe"))

-- Menu option crabSteering
        self.menuOption_crabSteering = self.checkDirt:clone()
        self.menuOption_crabSteering.target = g_engineStarter
        self.menuOption_crabSteering.id = "engineStarter_crabSteering"
        self.menuOption_crabSteering:setCallback("onClickCallback", "onMenuOptionChanged_crabSteering")

        local settingTitle = self.menuOption_crabSteering.elements[4]
        local toolTip = self.menuOption_crabSteering.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_crabSteering:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_crabSteering"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_crabSteering"))

-- Menu option dischargeable
        self.menuOption_dischargeable = self.checkDirt:clone()
        self.menuOption_dischargeable.target = g_engineStarter
        self.menuOption_dischargeable.id = "engineStarter_dischargeable"
        self.menuOption_dischargeable:setCallback("onClickCallback", "onMenuOptionChanged_dischargeable")

        local settingTitle = self.menuOption_dischargeable.elements[4]
        local toolTip = self.menuOption_dischargeable.elements[6]

		-- Set choices from value list
		local textTable = {}
		for _, option in ipairs(g_engineStarter.multiShared_values) do
			table.insert(textTable, option)
		end
		self.menuOption_dischargeable:setTexts(textTable)
		
        settingTitle:setText(g_i18n:getText("configuration_dischargeable"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_dischargeable"))

-- Menu option startForOthers
        self.menuOption_startForOthers = self.checkAutoMotorStart:clone()
        self.menuOption_startForOthers.target = g_engineStarter
        self.menuOption_startForOthers.id = "engineStarter_startForOthers"
        self.menuOption_startForOthers:setCallback("onClickCallback", "onMenuOptionChanged_startForOthers")

        local settingTitle = self.menuOption_startForOthers.elements[4]
        local toolTip = self.menuOption_startForOthers.elements[6]

		self.menuOption_startForOthers:setTexts({g_engineStarter.multiShared_values[1], g_engineStarter.multiShared_values[3]})
		
        settingTitle:setText(g_i18n:getText("setting_engineStarter_startForOthers"))
        toolTip:setText(g_i18n:getText("toolTip_engineStarter_startForOthers"))




-- Add elements to in-game menu
        local title = TextElement.new()
        title:applyProfile("settingsMenuSubtitle", true)
        title:setText(g_i18n:getText("title_engineStarter"))

        self.boxLayout:addElement(title)
        self.boxLayout:addElement(self.menuOption_ThrottleStart)
        self.boxLayout:addElement(self.menuOption_QuickStart)
        self.boxLayout:addElement(self.menuOption_ShowWarnings)
        self.boxLayout:addElement(self.menuOption_turnOnVehicle)
        -- self.boxLayout:addElement(self.menuOption_motorStopTimerDuration)
        self.boxLayout:addElement(self.menuOption_Steerable)
        self.boxLayout:addElement(self.menuOption_cylindered)
        self.boxLayout:addElement(self.menuOption_foldable)
        self.boxLayout:addElement(self.menuOption_foldMiddle)
        self.boxLayout:addElement(self.menuOption_pipe)
        self.boxLayout:addElement(self.menuOption_crabSteering)
        self.boxLayout:addElement(self.menuOption_dischargeable)
        self.boxLayout:addElement(self.menuOption_startForOthers)

		
		-- Save these for later, so we can toggle/disable them
		g_engineStarter.checkAutoMotorStart = self.checkAutoMotorStart 
		g_engineStarter.checkAutoMotorStart_backupTooltip = self.checkAutoMotorStart.elements[6].sourceText

		g_engineStarter.menuOption_QuickStart = self.menuOption_QuickStart -- save this for later too

        g_engineStarter.createdGUI = true -- So we don't keep adding the menu items
    end

	-- Load the current values into the settings menu, each time it loads
	self.menuOption_ThrottleStart:setIsChecked(g_engineStarter.isEnabled_ThrottleStart)
	self.menuOption_QuickStart:setIsChecked(g_engineStarter.isEnabled_QuickStart)
	self.menuOption_ShowWarnings:setIsChecked(g_engineStarter.isEnabled_ShowWarnings)
	self.menuOption_turnOnVehicle:setIsChecked(g_engineStarter.isEnabled_turnOnVehicle)
	-- self.menuOption_motorStopTimerDuration:setState(g_engineStarter.multi_motorStopTimerDuration)
	self.menuOption_Steerable:setState(g_engineStarter.multiShared_Steerable)
	self.menuOption_cylindered:setState(g_engineStarter.multiShared_cylindered)
	self.menuOption_foldable:setState(g_engineStarter.multiShared_foldable)
	self.menuOption_foldMiddle:setState(g_engineStarter.multiShared_foldMiddle)
	self.menuOption_pipe:setState(g_engineStarter.multiShared_pipe)
	self.menuOption_crabSteering:setState(g_engineStarter.multiShared_crabSteering)
	self.menuOption_dischargeable:setState(g_engineStarter.multiShared_dischargeable)
	self.menuOption_startForOthers:setIsChecked(g_engineStarter.isEnabled_Other)

	-- Set base-game AutoStart, and QuickStart to off and disable
	if (g_engineStarter.isEnabled_ThrottleStart) then
		self.checkAutoMotorStart:setIsChecked(false)
		g_engineStarter.checkAutoMotorStart.elements[6]:setText(g_i18n:getText("notice_AutoMotorStartDisabled"))
	else
		self.menuOption_QuickStart:setIsChecked(false)
		g_engineStarter.checkAutoMotorStart.elements[6]:setText(g_engineStarter.checkAutoMotorStart_backupTooltip)
	end

	self.checkAutoMotorStart:setDisabled(g_engineStarter.isEnabled_ThrottleStart)
	self.menuOption_QuickStart:setDisabled(not g_engineStarter.isEnabled_ThrottleStart)

end

--- Menu callbacks - called when menu option changes
function EngineStarter:onMenuOptionChanged_ThrottleStart(state)
    self.isEnabled_ThrottleStart = state == CheckedOptionElement.STATE_CHECKED
	
	-- Set base-game AutoStart and QuickStart to off and disable
	if (self.isEnabled_ThrottleStart) then
		self.checkAutoMotorStart:setIsChecked(false) -- Uncheck the menu item
		g_currentMission.missionInfo.automaticMotorStartEnabled = false -- Directly set the settings (since the menu item callback won't be called)
		g_engineStarter.checkAutoMotorStart.elements[6]:setText(g_i18n:getText("notice_AutoMotorStartDisabled")) -- Update the tooltip
		self.menuOption_QuickStart:setIsChecked(g_engineStarter.isEnabled_QuickStart) -- Force this to update to actual setting (since we pretend its off when we disable it)
	else
		self.menuOption_QuickStart:setIsChecked(false) -- uncheck the menu item (we don't care about the actual setting. It's just to highlight to players that they can't use it
		g_engineStarter.checkAutoMotorStart.elements[6]:setText(g_engineStarter.checkAutoMotorStart_backupTooltip) -- Restore that tooltop we changed
	end

	self.checkAutoMotorStart:setDisabled(self.isEnabled_ThrottleStart)
	self.menuOption_QuickStart:setDisabled(not self.isEnabled_ThrottleStart)
	
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_QuickStart(state)
    self.isEnabled_QuickStart = state == CheckedOptionElement.STATE_CHECKED
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_ShowWarnings(state)
    self.isEnabled_ShowWarnings = state == CheckedOptionElement.STATE_CHECKED
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_turnOnVehicle(state)
    self.isEnabled_turnOnVehicle = state == CheckedOptionElement.STATE_CHECKED
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_Steerable(state)
	self.multiShared_Steerable = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_cylindered(state)
	self.multiShared_cylindered = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_foldable(state)
	self.multiShared_foldable = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_foldMiddle(state)
	self.multiShared_foldMiddle = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_pipe(state)
	self.multiShared_pipe = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_crabSteering(state)
	self.multiShared_crabSteering = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_dischargeable(state)
	self.multiShared_dischargeable = state
	EngineStarter:saveSettings()
end

function EngineStarter:onMenuOptionChanged_startForOthers(state)
	self.isEnabled_Other = state == CheckedOptionElement.STATE_CHECKED
	EngineStarter:saveSettings()
end


-- function EngineStarter:onMenuOptionChanged_motorStopTimerDuration(state)
    -- self.multi_motorStopTimerDuration = state
	-- EngineStarter:saveSettings()
-- end
